package net.minecraft.block;

import java.util.List;

import net.minecraft.block.material.MapColor;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.IProperty;
import net.minecraft.block.properties.PropertyBool;
import net.minecraft.block.properties.PropertyEnum;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.init.Blocks;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.util.IStringSerializable;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;

public class BlockDirt extends Block {
	public static final PropertyEnum<BlockDirt.DirtType> VARIANT = PropertyEnum.<BlockDirt.DirtType> create("variant", BlockDirt.DirtType.class);
	public static final PropertyBool SNOWY = PropertyBool.create("snowy");

	protected BlockDirt() {
		super(Material.GROUND);
		this.setDefaultState(this.blockState.getBaseState().withProperty(VARIANT, BlockDirt.DirtType.DIRT).withProperty(SNOWY, Boolean.valueOf(false)));
		this.setCreativeTab(CreativeTabs.BUILDING_BLOCKS);
	}

	/**
	 * Get the MapColor for this Block and the given BlockState
	 */
	@Override
	public MapColor getMapColor(IBlockState state) {
		return state.getValue(VARIANT).getColor();
	}

	/**
	 * Get the actual Block state of this Block at the given position. This applies properties not visible in the metadata, such as fence connections.
	 */
	@Override
	public IBlockState getActualState(IBlockState state, IBlockAccess worldIn, BlockPos pos) {
		if (state.getValue(VARIANT) == BlockDirt.DirtType.PODZOL) {
			Block block = worldIn.getBlockState(pos.up()).getBlock();
			state = state.withProperty(SNOWY, Boolean.valueOf((block == Blocks.SNOW) || (block == Blocks.SNOW_LAYER)));
		}

		return state;
	}

	/**
	 * returns a list of blocks with the same ID, but different meta (eg: wood returns 4 blocks)
	 */
	@Override
	public void getSubBlocks(Item itemIn, CreativeTabs tab, List<ItemStack> list) {
		list.add(new ItemStack(this, 1, BlockDirt.DirtType.DIRT.getMetadata()));
		list.add(new ItemStack(this, 1, BlockDirt.DirtType.COARSE_DIRT.getMetadata()));
		list.add(new ItemStack(this, 1, BlockDirt.DirtType.PODZOL.getMetadata()));
	}

	@Override
	public ItemStack getItem(World worldIn, BlockPos pos, IBlockState state) {
		return new ItemStack(this, 1, state.getValue(VARIANT).getMetadata());
	}

	/**
	 * Convert the given metadata into a BlockState for this Block
	 */
	@Override
	public IBlockState getStateFromMeta(int meta) {
		return this.getDefaultState().withProperty(VARIANT, BlockDirt.DirtType.byMetadata(meta));
	}

	/**
	 * Convert the BlockState into the correct metadata value
	 */
	@Override
	public int getMetaFromState(IBlockState state) {
		return state.getValue(VARIANT).getMetadata();
	}

	@Override
	protected BlockStateContainer createBlockState() {
		return new BlockStateContainer(this, new IProperty[] { VARIANT, SNOWY });
	}

	/**
	 * Gets the metadata of the item this Block can drop. This method is called when the block gets destroyed. It returns the metadata of the dropped item based on the old metadata of the block.
	 */
	@Override
	public int damageDropped(IBlockState state) {
		BlockDirt.DirtType blockdirt$dirttype = state.getValue(VARIANT);

		if (blockdirt$dirttype == BlockDirt.DirtType.PODZOL) {
			blockdirt$dirttype = BlockDirt.DirtType.DIRT;
		}

		return blockdirt$dirttype.getMetadata();
	}

	public static enum DirtType implements IStringSerializable {
		DIRT(0, "dirt", "default", MapColor.DIRT), COARSE_DIRT(1, "coarse_dirt", "coarse", MapColor.DIRT), PODZOL(2, "podzol", MapColor.OBSIDIAN);

		private static final BlockDirt.DirtType[] METADATA_LOOKUP = new BlockDirt.DirtType[values().length];
		private final int metadata;
		private final String name;
		private final String unlocalizedName;
		private final MapColor color;

		private DirtType(int metadataIn, String nameIn, MapColor color) {
			this(metadataIn, nameIn, nameIn, color);
		}

		private DirtType(int metadataIn, String nameIn, String unlocalizedNameIn, MapColor color) {
			this.metadata = metadataIn;
			this.name = nameIn;
			this.unlocalizedName = unlocalizedNameIn;
			this.color = color;
		}

		public int getMetadata() {
			return this.metadata;
		}

		public String getUnlocalizedName() {
			return this.unlocalizedName;
		}

		public MapColor getColor() {
			return this.color;
		}

		@Override
		public String toString() {
			return this.name;
		}

		public static BlockDirt.DirtType byMetadata(int metadata) {
			if ((metadata < 0) || (metadata >= METADATA_LOOKUP.length)) {
				metadata = 0;
			}

			return METADATA_LOOKUP[metadata];
		}

		@Override
		public String getName() {
			return this.name;
		}

		static {
			for (BlockDirt.DirtType blockdirt$dirttype : values()) {
				METADATA_LOOKUP[blockdirt$dirttype.getMetadata()] = blockdirt$dirttype;
			}
		}
	}
}
