package net.minecraft.entity.monster;

import java.util.Calendar;

import javax.annotation.Nullable;

import net.minecraft.block.Block;
import net.minecraft.enchantment.EnchantmentHelper;
import net.minecraft.entity.*;
import net.minecraft.entity.ai.*;
import net.minecraft.entity.item.EntityBoat;
import net.minecraft.entity.passive.EntityWolf;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.projectile.EntityArrow;
import net.minecraft.entity.projectile.EntityTippedArrow;
import net.minecraft.init.*;
import net.minecraft.inventory.EntityEquipmentSlot;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.network.datasync.DataParameter;
import net.minecraft.network.datasync.DataSerializers;
import net.minecraft.network.datasync.EntityDataManager;
import net.minecraft.potion.PotionEffect;
import net.minecraft.stats.AchievementList;
import net.minecraft.util.DamageSource;
import net.minecraft.util.EnumHand;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.SoundEvent;
import net.minecraft.util.datafix.DataFixer;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.MathHelper;
import net.minecraft.world.DifficultyInstance;
import net.minecraft.world.EnumDifficulty;
import net.minecraft.world.World;
import net.minecraft.world.WorldProviderHell;
import net.minecraft.world.biome.Biome;
import net.minecraft.world.biome.BiomeSnow;

public class EntitySkeleton extends EntityMob implements IRangedAttackMob {
	private static final DataParameter<Integer> SKELETON_VARIANT = EntityDataManager.<Integer> createKey(EntitySkeleton.class, DataSerializers.VARINT);
	private static final DataParameter<Boolean> SWINGING_ARMS = EntityDataManager.<Boolean> createKey(EntitySkeleton.class, DataSerializers.BOOLEAN);
	private final EntityAIAttackRangedBow aiArrowAttack = new EntityAIAttackRangedBow(this, 1.0D, 20, 15.0F);
	private final EntityAIAttackMelee aiAttackOnCollide = new EntityAIAttackMelee(this, 1.2D, false) {
		@Override
		public void resetTask() {
			super.resetTask();
			EntitySkeleton.this.setSwingingArms(false);
		}

		@Override
		public void startExecuting() {
			super.startExecuting();
			EntitySkeleton.this.setSwingingArms(true);
		}
	};

	public EntitySkeleton(World worldIn) {
		super(worldIn);
		this.setCombatTask();
	}

	@Override
	protected void initEntityAI() {
		this.tasks.addTask(1, new EntityAISwimming(this));
		this.tasks.addTask(2, new EntityAIRestrictSun(this));
		this.tasks.addTask(3, new EntityAIFleeSun(this, 1.0D));
		this.tasks.addTask(3, new EntityAIAvoidEntity(this, EntityWolf.class, 6.0F, 1.0D, 1.2D));
		this.tasks.addTask(5, new EntityAIWander(this, 1.0D));
		this.tasks.addTask(6, new EntityAIWatchClosest(this, EntityPlayer.class, 8.0F));
		this.tasks.addTask(6, new EntityAILookIdle(this));
		this.targetTasks.addTask(1, new EntityAIHurtByTarget(this, false, new Class[0]));
		this.targetTasks.addTask(2, new EntityAINearestAttackableTarget(this, EntityPlayer.class, true));
		this.targetTasks.addTask(3, new EntityAINearestAttackableTarget(this, EntityIronGolem.class, true));
	}

	@Override
	protected void applyEntityAttributes() {
		super.applyEntityAttributes();
		this.getEntityAttribute(SharedMonsterAttributes.MOVEMENT_SPEED).setBaseValue(0.25D);
	}

	@Override
	protected void entityInit() {
		super.entityInit();
		this.dataManager.register(SKELETON_VARIANT, Integer.valueOf(SkeletonType.NORMAL.func_190135_a()));
		this.dataManager.register(SWINGING_ARMS, Boolean.valueOf(false));
	}

	@Override
	protected SoundEvent getAmbientSound() {
		return this.func_189771_df().func_190136_d();
	}

	@Override
	protected SoundEvent getHurtSound() {
		return this.func_189771_df().func_190132_e();
	}

	@Override
	protected SoundEvent getDeathSound() {
		return this.func_189771_df().func_190133_f();
	}

	@Override
	protected void playStepSound(BlockPos pos, Block blockIn) {
		SoundEvent soundevent = this.func_189771_df().func_190131_g();
		this.playSound(soundevent, 0.15F, 1.0F);
	}

	@Override
	public boolean attackEntityAsMob(Entity entityIn) {
		if (super.attackEntityAsMob(entityIn)) {
			if ((this.func_189771_df() == SkeletonType.WITHER) && (entityIn instanceof EntityLivingBase)) {
				((EntityLivingBase) entityIn).addPotionEffect(new PotionEffect(MobEffects.WITHER, 200));
			}

			return true;
		} else {
			return false;
		}
	}

	/**
	 * Get this Entity's EnumCreatureAttribute
	 */
	@Override
	public EnumCreatureAttribute getCreatureAttribute() {
		return EnumCreatureAttribute.UNDEAD;
	}

	/**
	 * Called frequently so the entity can update its state every tick as required. For example, zombies and skeletons use this to react to sunlight and start to burn.
	 */
	@Override
	public void onLivingUpdate() {
		if (this.worldObj.isDaytime() && !this.worldObj.isRemote) {
			float f = this.getBrightness(1.0F);
			BlockPos blockpos = this.getRidingEntity() instanceof EntityBoat ? (new BlockPos(this.posX, Math.round(this.posY), this.posZ)).up()
					: new BlockPos(this.posX, Math.round(this.posY), this.posZ);

			if ((f > 0.5F) && ((this.rand.nextFloat() * 30.0F) < ((f - 0.4F) * 2.0F)) && this.worldObj.canSeeSky(blockpos)) {
				boolean flag = true;
				ItemStack itemstack = this.getItemStackFromSlot(EntityEquipmentSlot.HEAD);

				if (itemstack != null) {
					if (itemstack.isItemStackDamageable()) {
						itemstack.setItemDamage(itemstack.getItemDamage() + this.rand.nextInt(2));

						if (itemstack.getItemDamage() >= itemstack.getMaxDamage()) {
							this.renderBrokenItemStack(itemstack);
							this.setItemStackToSlot(EntityEquipmentSlot.HEAD, (ItemStack) null);
						}
					}

					flag = false;
				}

				if (flag) {
					this.setFire(8);
				}
			}
		}

		if (this.worldObj.isRemote) {
			this.func_189769_b(this.func_189771_df());
		}

		super.onLivingUpdate();
	}

	/**
	 * Handles updating while being ridden by an entity
	 */
	@Override
	public void updateRidden() {
		super.updateRidden();

		if (this.getRidingEntity() instanceof EntityCreature) {
			EntityCreature entitycreature = (EntityCreature) this.getRidingEntity();
			this.renderYawOffset = entitycreature.renderYawOffset;
		}
	}

	/**
	 * Called when the mob's health reaches 0.
	 */
	@Override
	public void onDeath(DamageSource cause) {
		super.onDeath(cause);

		if ((cause.getSourceOfDamage() instanceof EntityArrow) && (cause.getEntity() instanceof EntityPlayer)) {
			EntityPlayer entityplayer = (EntityPlayer) cause.getEntity();
			double d0 = entityplayer.posX - this.posX;
			double d1 = entityplayer.posZ - this.posZ;

			if (((d0 * d0) + (d1 * d1)) >= 2500.0D) {
				entityplayer.addStat(AchievementList.SNIPE_SKELETON);
			}
		} else if ((cause.getEntity() instanceof EntityCreeper) && ((EntityCreeper) cause.getEntity()).getPowered() && ((EntityCreeper) cause.getEntity()).isAIEnabled()) {
			((EntityCreeper) cause.getEntity()).incrementDroppedSkulls();
			this.entityDropItem(new ItemStack(Items.SKULL, 1, this.func_189771_df() == SkeletonType.WITHER ? 1 : 0), 0.0F);
		}
	}

	@Override
	@Nullable
	protected ResourceLocation getLootTable() {
		return this.func_189771_df().func_190129_c();
	}

	/**
	 * Gives armor or weapon for entity based on given DifficultyInstance
	 */
	@Override
	protected void setEquipmentBasedOnDifficulty(DifficultyInstance difficulty) {
		super.setEquipmentBasedOnDifficulty(difficulty);
		this.setItemStackToSlot(EntityEquipmentSlot.MAINHAND, new ItemStack(Items.BOW));
	}

	@Override
	@Nullable

	/**
	 * Called only once on an entity when first time spawned, via egg, mob spawner, natural spawning etc, but not called when entity is reloaded from nbt. Mainly used for initializing attributes and inventory
	 */
	public IEntityLivingData onInitialSpawn(DifficultyInstance difficulty, @Nullable IEntityLivingData livingdata) {
		livingdata = super.onInitialSpawn(difficulty, livingdata);

		if ((this.worldObj.provider instanceof WorldProviderHell) && (this.getRNG().nextInt(5) > 0)) {
			this.tasks.addTask(4, this.aiAttackOnCollide);
			this.func_189768_a(SkeletonType.WITHER);
			this.setItemStackToSlot(EntityEquipmentSlot.MAINHAND, new ItemStack(Items.STONE_SWORD));
			this.getEntityAttribute(SharedMonsterAttributes.ATTACK_DAMAGE).setBaseValue(4.0D);
		} else {
			Biome biome = this.worldObj.getBiomeGenForCoords(new BlockPos(this));

			if ((biome instanceof BiomeSnow) && this.worldObj.canSeeSky(new BlockPos(this)) && (this.rand.nextInt(5) != 0)) {
				this.func_189768_a(SkeletonType.STRAY);
			}

			this.tasks.addTask(4, this.aiArrowAttack);
			this.setEquipmentBasedOnDifficulty(difficulty);
			this.setEnchantmentBasedOnDifficulty(difficulty);
		}

		this.setCanPickUpLoot(this.rand.nextFloat() < (0.55F * difficulty.getClampedAdditionalDifficulty()));

		if (this.getItemStackFromSlot(EntityEquipmentSlot.HEAD) == null) {
			Calendar calendar = this.worldObj.getCurrentDate();

			if (((calendar.get(2) + 1) == 10) && (calendar.get(5) == 31) && (this.rand.nextFloat() < 0.25F)) {
				this.setItemStackToSlot(EntityEquipmentSlot.HEAD, new ItemStack(this.rand.nextFloat() < 0.1F ? Blocks.LIT_PUMPKIN : Blocks.PUMPKIN));
				this.inventoryArmorDropChances[EntityEquipmentSlot.HEAD.getIndex()] = 0.0F;
			}
		}

		return livingdata;
	}

	/**
	 * sets this entity's combat AI.
	 */
	public void setCombatTask() {
		if ((this.worldObj != null) && !this.worldObj.isRemote) {
			this.tasks.removeTask(this.aiAttackOnCollide);
			this.tasks.removeTask(this.aiArrowAttack);
			ItemStack itemstack = this.getHeldItemMainhand();

			if ((itemstack != null) && (itemstack.getItem() == Items.BOW)) {
				int i = 20;

				if (this.worldObj.getDifficulty() != EnumDifficulty.HARD) {
					i = 40;
				}

				this.aiArrowAttack.func_189428_b(i);
				this.tasks.addTask(4, this.aiArrowAttack);
			} else {
				this.tasks.addTask(4, this.aiAttackOnCollide);
			}
		}
	}

	/**
	 * Attack the specified entity using a ranged attack.
	 */
	@Override
	public void attackEntityWithRangedAttack(EntityLivingBase target, float p_82196_2_) {
		EntityTippedArrow entitytippedarrow = new EntityTippedArrow(this.worldObj, this);
		double d0 = target.posX - this.posX;
		double d1 = (target.getEntityBoundingBox().minY + target.height / 3.0F) - entitytippedarrow.posY;
		double d2 = target.posZ - this.posZ;
		double d3 = MathHelper.sqrt_double((d0 * d0) + (d2 * d2));
		entitytippedarrow.setThrowableHeading(d0, d1 + (d3 * 0.20000000298023224D), d2, 1.6F, 14 - (this.worldObj.getDifficulty().getDifficultyId() * 4));
		int i = EnchantmentHelper.getMaxEnchantmentLevel(Enchantments.POWER, this);
		int j = EnchantmentHelper.getMaxEnchantmentLevel(Enchantments.PUNCH, this);
		DifficultyInstance difficultyinstance = this.worldObj.getDifficultyForLocation(new BlockPos(this));
		entitytippedarrow.setDamage(p_82196_2_ * 2.0F + (this.rand.nextGaussian() * 0.25D) + this.worldObj.getDifficulty().getDifficultyId() * 0.11F);

		if (i > 0) {
			entitytippedarrow.setDamage(entitytippedarrow.getDamage() + (i * 0.5D) + 0.5D);
		}

		if (j > 0) {
			entitytippedarrow.setKnockbackStrength(j);
		}

		boolean flag = (this.isBurning() && difficultyinstance.func_190083_c() && this.rand.nextBoolean()) || (this.func_189771_df() == SkeletonType.WITHER);
		flag = flag || (EnchantmentHelper.getMaxEnchantmentLevel(Enchantments.FLAME, this) > 0);

		if (flag) {
			entitytippedarrow.setFire(100);
		}

		ItemStack itemstack = this.getHeldItem(EnumHand.OFF_HAND);

		if ((itemstack != null) && (itemstack.getItem() == Items.TIPPED_ARROW)) {
			entitytippedarrow.setPotionEffect(itemstack);
		} else if (this.func_189771_df() == SkeletonType.STRAY) {
			entitytippedarrow.addEffect(new PotionEffect(MobEffects.SLOWNESS, 600));
		}

		this.playSound(SoundEvents.ENTITY_SKELETON_SHOOT, 1.0F, 1.0F / ((this.getRNG().nextFloat() * 0.4F) + 0.8F));
		this.worldObj.spawnEntityInWorld(entitytippedarrow);
	}

	public SkeletonType func_189771_df() {
		return SkeletonType.func_190134_a(this.dataManager.get(SKELETON_VARIANT).intValue());
	}

	public void func_189768_a(SkeletonType p_189768_1_) {
		this.dataManager.set(SKELETON_VARIANT, Integer.valueOf(p_189768_1_.func_190135_a()));
		this.isImmuneToFire = p_189768_1_ == SkeletonType.WITHER;
		this.func_189769_b(p_189768_1_);
	}

	private void func_189769_b(SkeletonType p_189769_1_) {
		if (p_189769_1_ == SkeletonType.WITHER) {
			this.setSize(0.7F, 2.4F);
		} else {
			this.setSize(0.6F, 1.99F);
		}
	}

	public static void func_189772_b(DataFixer p_189772_0_) {
		EntityLiving.func_189752_a(p_189772_0_, "Skeleton");
	}

	/**
	 * (abstract) Protected helper method to read subclass entity data from NBT.
	 */
	@Override
	public void readEntityFromNBT(NBTTagCompound compound) {
		super.readEntityFromNBT(compound);

		if (compound.hasKey("SkeletonType", 99)) {
			int i = compound.getByte("SkeletonType");
			this.func_189768_a(SkeletonType.func_190134_a(i));
		}

		this.setCombatTask();
	}

	/**
	 * (abstract) Protected helper method to write subclass entity data to NBT.
	 */
	@Override
	public void writeEntityToNBT(NBTTagCompound compound) {
		super.writeEntityToNBT(compound);
		compound.setByte("SkeletonType", (byte) this.func_189771_df().func_190135_a());
	}

	@Override
	public void setItemStackToSlot(EntityEquipmentSlot slotIn, @Nullable ItemStack stack) {
		super.setItemStackToSlot(slotIn, stack);

		if (!this.worldObj.isRemote && (slotIn == EntityEquipmentSlot.MAINHAND)) {
			this.setCombatTask();
		}
	}

	@Override
	public float getEyeHeight() {
		return this.func_189771_df() == SkeletonType.WITHER ? 2.1F : 1.74F;
	}

	/**
	 * Returns the Y Offset of this entity.
	 */
	@Override
	public double getYOffset() {
		return -0.35D;
	}

	public boolean isSwingingArms() {
		return this.dataManager.get(SWINGING_ARMS).booleanValue();
	}

	public void setSwingingArms(boolean swingingArms) {
		this.dataManager.set(SWINGING_ARMS, Boolean.valueOf(swingingArms));
	}
}
